"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.baseGte = baseGte;
exports.eq = eq;
// predicate should return true if we should go right.
function baseLte(inclusive, array, value, cmp) {
    var len = array.length;
    if (len === 0 || !(inclusive ? cmp(array[0], value) <= 0 : cmp(array[0], value) < 0)) {
        return -1;
    }

    var l = 0;
    var r = len;
    while (r - l > 1) {
        // In our case, r + l <= 128 so we don't need
        // to worry about overflow here
        var mid = r + l >>> 1;
        var item = array[mid];
        if (inclusive ? cmp(item, value) <= 0 : cmp(item, value) < 0) {
            // value at `mid` is less than or equal to `value`, go right.
            l = mid;
        } else {
            // if value at `mid` is larger than `value`, go left.
            r = mid;
        }
    }

    return l;
}

var lte = exports.lte = baseLte.bind(null, true);
var lt = exports.lt = baseLte.bind(null, false);

// predicate should return true if we should go left
function baseGte(inclusive, array, value, cmp) {
    var len = array.length;
    if (len === 0 || !(inclusive ? cmp(array[len - 1], value) >= 0 : cmp(array[len - 1], value) > 0)) return len;
    var l = -1;
    var r = len - 1;
    while (r - l > 1) {
        var mid = r + l >>> 1;
        var item = array[mid];
        if (inclusive ? cmp(item, value) >= 0 : cmp(item, value) > 0) {
            // value at `mid` is less than or equal to `value`, go right.
            r = mid;
        } else {
            // if value at `mid` is larger than `value`, go left.
            l = mid;
        }
    }

    return r;
}

var gte = exports.gte = baseGte.bind(null, true);
var gt = exports.gt = baseGte.bind(null, false);

function eq(array, value, cmp) {
    var idx = lte(array, value, cmp);
    if (idx !== -1 && cmp(array[idx], value) === 0) {
        return idx;
    }
    return -1;
}

exports.default = {
    lt: lt,
    gt: gt,
    lte: lte,
    gte: gte,
    eq: eq
};